@@ -27,7 +27,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: testutil.py 1112 2007-02-17 23:22:17Z pronovic $
+# Revision : $Id: testutil.py 1212 2007-07-10 03:08:39Z pronovic $
 # Purpose  : Provides unit-testing utilities.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -72,6 +72,7 @@ import getpass
 import random
 import string
 import platform
+import logging
 from StringIO import StringIO
 
 from CedarBackup2.util import encodePath
@@ -81,6 +82,29 @@ from CedarBackup2.util import encodePath
 # Public functions
 ########################################################################
 
+##############################
+# setupDebugLogger() function
+##############################
+
+def setupDebugLogger():
+   """
+   Sets up a screen logger for debugging purposes.
+
+   Normally, the CLI functionality configures the logger so that
+   things get written to the right place.  However, for debugging
+   it's sometimes nice to just get everything -- debug information
+   and output -- dumped to the screen.  This function takes care
+   of that.
+   """
+   logger = logging.getLogger("CedarBackup2")
+   logger.setLevel(logging.DEBUG)    # let the logger see all messages
+   formatter = logging.Formatter(fmt="%(message)s")
+   handler = logging.StreamHandler(strm=sys.stdout)
+   handler.setFormatter(formatter)
+   handler.setLevel(logging.DEBUG)
+   logger.addHandler(handler)
+
+
 ###########################
 # findResources() function
 ###########################
@@ -294,7 +318,7 @@ def failUnlessAssignRaises(testCase, exception, object, property, value):
    @param property: Name of the property, as a string.
    @param value: Value that is to be assigned to the property.
 
-   @see: L{unittest.TestCase.failUnlessRaises}
+   @see: C{unittest.TestCase.failUnlessRaises}
    """
    missed = False
    instead = None
