@@ -30,7 +30,7 @@
 # Author   : Kenneth J. Pronovici <pronovic@ieee.org>
 # Language : Python (>= 2.3)
 # Project  : Cedar Backup, release 2
-# Revision : $Id: util.py 1138 2007-02-24 04:15:50Z pronovic $
+# Revision : $Id: util.py 1213 2007-07-10 03:10:02Z pronovic $
 # Purpose  : Provides general-purpose utilities.
 #
 # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
@@ -43,7 +43,7 @@
 Provides general-purpose utilities. 
 
 @sort: AbsolutePathList, ObjectTypeList, RestrictedValueList, RegexMatchList,
-       _Vertex, DirectedGraph, PathResolverSingleton, 
+       RegexList, _Vertex, DirectedGraph, PathResolverSingleton, 
        sortDict, convertSize, getUidGid, changeOwnership, splitCommandLine,
        resolveCommand, executeCommand, calculateFileAge, encodePath, nullDevice,
        deriveDayOfWeek, isStartOfWeek, buildNormalizedPath, 
@@ -281,7 +281,7 @@ class AbsolutePathList(UnorderedList):
       @raise ValueError: If item is not an absolute path.
       """
       if not os.path.isabs(item):
-         raise ValueError("Item must be an absolute path.")
+         raise ValueError("Not an absolute path: [%s]" % item)
       list.append(self, encodePath(item))
 
    def insert(self, index, item):
@@ -290,7 +290,7 @@ class AbsolutePathList(UnorderedList):
       @raise ValueError: If item is not an absolute path.
       """
       if not os.path.isabs(item):
-         raise ValueError("Item must be an absolute path.")
+         raise ValueError("Not an absolute path: [%s]" % item)
       list.insert(self, index, encodePath(item))
 
    def extend(self, seq):
@@ -300,7 +300,7 @@ class AbsolutePathList(UnorderedList):
       """
       for item in seq:
          if not os.path.isabs(item):
-            raise ValueError("All items must be absolute paths.")
+            raise ValueError("Not an absolute path: [%s]" % item)
       for item in seq:
          list.append(self, encodePath(item))
 
@@ -389,12 +389,15 @@ class RestrictedContentList(UnorderedList):
    exceptions or other problems.
    """
    
-   def __init__(self, valuesList, valuesDescr):
+   def __init__(self, valuesList, valuesDescr, prefix=None):
       """
       Initializes a list restricted to containing certain values.
       @param valuesList: List of valid values.
       @param valuesDescr: Short string describing list of values.
+      @param prefix: Prefix to use in error messages (None results in prefix "Item")
       """
+      self.prefix = "Item"
+      if prefix is not None: self.prefix = prefix
       self.valuesList = valuesList
       self.valuesDescr = valuesDescr
 
@@ -404,7 +407,7 @@ class RestrictedContentList(UnorderedList):
       @raise ValueError: If item is not in the values list.
       """
       if item not in self.valuesList:
-         raise ValueError("Item must be one of values in %s." % self.valuesDescr)
+         raise ValueError("%s must be one of the values in %s." % (self.prefix, self.valuesDescr))
       list.append(self, item)
 
    def insert(self, index, item):
@@ -413,7 +416,7 @@ class RestrictedContentList(UnorderedList):
       @raise ValueError: If item is not in the values list.
       """
       if item not in self.valuesList:
-         raise ValueError("Item must be one of values in %s." % self.valuesDescr)
+         raise ValueError("%s must be one of the values in %s." % (self.prefix, self.valuesDescr))
       list.insert(self, index, item)
 
    def extend(self, seq):
@@ -423,7 +426,7 @@ class RestrictedContentList(UnorderedList):
       """
       for item in seq:
          if item not in self.valuesList:
-            raise ValueError("Item must be one of values in %s." % self.valuesDescr)
+            raise ValueError("%s must be one of the values in %s." % (self.prefix, self.valuesDescr))
       list.extend(self, seq)
 
 
@@ -450,12 +453,15 @@ class RegexMatchList(UnorderedList):
    likely get either TypeError or AttributeError exceptions as a result.
    """
    
-   def __init__(self, valuesRegex, emptyAllowed=True):
+   def __init__(self, valuesRegex, emptyAllowed=True, prefix=None):
       """
       Initializes a list restricted to containing certain values.
       @param valuesRegex: Regular expression that must be matched, as a string
       @param emptyAllowed: Indicates whether empty or None values are allowed.
+      @param prefix: Prefix to use in error messages (None results in prefix "Item")
       """
+      self.prefix = "Item"
+      if prefix is not None: self.prefix = prefix
       self.valuesRegex = valuesRegex
       self.emptyAllowed = emptyAllowed
       self.pattern = re.compile(self.valuesRegex)
@@ -468,9 +474,9 @@ class RegexMatchList(UnorderedList):
       @raise ValueError: If item does not match the configured regular expression
       """
       if item is None or (not self.emptyAllowed and item == ""):
-         raise ValueError("Item must be non-empty.")
+         raise ValueError("%s cannot be empty." % self.prefix)
       if not self.pattern.search(item):
-         raise ValueError("Item must match regular expression [%s]." % self.valuesRegex)
+         raise ValueError("%s is not valid: [%s]" % (self.prefix, item))
       list.append(self, item)
 
    def insert(self, index, item):
@@ -481,9 +487,9 @@ class RegexMatchList(UnorderedList):
       @raise ValueError: If item does not match the configured regular expression
       """
       if item is None or (not self.emptyAllowed and item == ""):
-         raise ValueError("Item must be non-empty.")
+         raise ValueError("%s cannot be empty." % self.prefix)
       if not self.pattern.search(item):
-         raise ValueError("Item must match regular expression [%s]." % self.valuesRegex)
+         raise ValueError("%s is not valid [%s]" % (self.prefix, item))
       list.insert(self, index, item)
 
    def extend(self, seq):
@@ -495,12 +501,63 @@ class RegexMatchList(UnorderedList):
       """
       for item in seq:
          if item is None or (not self.emptyAllowed and item == ""):
-            raise ValueError("Item must be non-empty.")
+            raise ValueError("%s cannot be empty.", self.prefix)
          if not self.pattern.search(item):
-            raise ValueError("Item must match regular expression [%s]." % self.valuesRegex)
+            raise ValueError("%s is not valid: [%s]" % (self.prefix, item))
       list.extend(self, seq)
 
 
+########################################################################
+# RegexList class definition
+########################################################################
+
+class RegexList(UnorderedList):
+
+   """
+   Class representing a list of valid regular expression strings.
+
+   This is an unordered list.
+
+   We override the C{append}, C{insert} and C{extend} methods to ensure that
+   any item added to the list is a valid regular expression.
+   """
+
+   def append(self, item):
+      """
+      Overrides the standard C{append} method.
+      @raise ValueError: If item is not an absolute path.
+      """
+      try:
+         re.compile(item)
+      except re.error:
+         raise ValueError("Not a valid regular expression: [%s]" % item)
+      list.append(self, item)
+
+   def insert(self, index, item):
+      """
+      Overrides the standard C{insert} method.
+      @raise ValueError: If item is not an absolute path.
+      """
+      try:
+         re.compile(item)
+      except re.error:
+         raise ValueError("Not a valid regular expression: [%s]" % item)
+      list.insert(self, index, item)
+
+   def extend(self, seq):
+      """
+      Overrides the standard C{insert} method.
+      @raise ValueError: If any item is not an absolute path.
+      """
+      for item in seq:
+         try:
+            re.compile(item)
+         except re.error:
+            raise ValueError("Not a valid regular expression: [%s]" % item)
+      for item in seq:
+         list.append(self, item)
+
+
 ########################################################################
 # Directed graph implementation
 ########################################################################
@@ -797,7 +854,7 @@ if _PIPE_IMPLEMENTATION == "subprocess.Popen":
 
       All of this functionality is provided (in Python 2.4 or later) by the
       C{subprocess.Popen} class, so when that class is available, we'll use it.
-      Otherwise, there's another implementation based on L{popen2.Popen4},
+      Otherwise, there's another implementation based on C{popen2.Popen4},
       which unfortunately only works on UNIX platforms.
       """
       def __init__(self, cmd, bufsize=-1, ignoreStderr=False):
@@ -945,6 +1002,8 @@ def convertSize(size, fromUnit, toUnit):
    @return: Number converted to new unit, as a float.
    @raise ValueError: If one of the units is invalid.
    """
+   if size is None:
+      raise ValueError("Cannot convert size of None.")
    if fromUnit == UNIT_BYTES:
       byteSize = float(size)
    elif fromUnit == UNIT_KBYTES:
@@ -956,7 +1015,7 @@ def convertSize(size, fromUnit, toUnit):
    elif fromUnit == UNIT_SECTORS:
       byteSize = float(size) * BYTES_PER_SECTOR
    else:
-      raise ValueError("Unknown 'from' unit %d." % fromUnit)
+      raise ValueError("Unknown 'from' unit %s." % fromUnit)
    if toUnit == UNIT_BYTES:
       return byteSize
    elif toUnit == UNIT_KBYTES:
@@ -968,7 +1027,7 @@ def convertSize(size, fromUnit, toUnit):
    elif toUnit == UNIT_SECTORS:
       return byteSize / BYTES_PER_SECTOR
    else:
-      raise ValueError("Unknown 'to' unit %d." % toUnit)
+      raise ValueError("Unknown 'to' unit %s." % toUnit)
 
 
 ##########################
@@ -981,7 +1040,7 @@ def displayBytes(bytes, digits=2):
 
    It's rather difficult to look at a number like "72372224 bytes" and get any
    meaningful information out of it.  It would be more useful to see something
-   like "72.37 MB".  That's what this function does.  Any time you want to display
+   like "69.02 MB".  That's what this function does.  Any time you want to display
    a byte value, i.e.::
 
       print "Size: %s bytes" % bytes
@@ -992,7 +1051,9 @@ def displayBytes(bytes, digits=2):
 
    What comes out will be sensibly formatted.  The indicated number of digits
    will be listed after the decimal point, rounded based on whatever rules are
-   used by Python's standard C{%f} string format specifier.
+   used by Python's standard C{%f} string format specifier. (Values less than 1
+   kB will be listed in bytes and will not have a decimal point, since the
+   concept of a fractional byte is nonsensical.)
 
    @param bytes: Byte quantity.
    @type bytes: Integer number of bytes.
@@ -1002,6 +1063,8 @@ def displayBytes(bytes, digits=2):
 
    @return: String, formatted for sensible display.
    """
+   if(bytes is None):
+      raise ValueError("Cannot display byte value of None.")
    bytes = float(bytes)
    if math.fabs(bytes) < BYTES_PER_KBYTE:
       format = "%.0f bytes"
@@ -1159,8 +1222,12 @@ def splitCommandLine(commandLine):
    @param commandLine: Command line string
    @type commandLine: String, i.e. "cback --verbose stage store"
 
-   @return: List of arguments, suitable for passing to L{popen2}.
+   @return: List of arguments, suitable for passing to C{popen2}.
+
+   @raise ValueError: If the command line is None.
    """
+   if commandLine is None:
+      raise ValueError("Cannot split command line of None.")
    fields = re.findall('[^ "]+|"[^"]+"', commandLine)
    fields = map(lambda field: field.replace('"', ''), fields)
    return fields
@@ -1215,8 +1282,8 @@ def executeCommand(command, args, returnOutput=False, ignoreStderr=False, doNotL
    """
    Executes a shell command, hopefully in a safe way.
 
-   This function exists to replace direct calls to L{os.popen()} in the Cedar
-   Backup code.  It's not safe to call a function such as L{os.popen()} with
+   This function exists to replace direct calls to C{os.popen} in the Cedar
+   Backup code.  It's not safe to call a function such as C{os.popen()} with
    untrusted arguments, since that can cause problems if the string contains
    non-safe variables or other constructs (imagine that the argument is
    C{$WHATEVER}, but C{$WHATEVER} contains something like C{"; rm -fR ~/;
@@ -1228,7 +1295,7 @@ def executeCommand(command, args, returnOutput=False, ignoreStderr=False, doNotL
 
    Under the normal case, this function will return a tuple of C{(status,
    None)} where the status is the wait-encoded return status of the call per
-   the L{popen2.Popen4} documentation.  If C{returnOutput} is passed in as
+   the C{popen2.Popen4} documentation.  If C{returnOutput} is passed in as
    C{True}, the function will return a tuple of C{(status, output)} where
    C{output} is a list of strings, one entry per line in the output from the
    command.  Output is always logged to the C{outputLogger.info()} target,
@@ -1669,21 +1736,43 @@ def buildNormalizedPath(path):
    """
    Returns a "normalized" path based on a path name.
 
-   A "normalized" path has its leading C{'/'} or C{'.'} characters removed, and
-   then converts all remaining whitespace and C{'/'} characters to the C{'_'}
-   character.   As a special case, the absolute path C{/} will be normalized to
-   just C{'-'}.
+   A normalized path is a representation of a path that is also a valid file
+   name.  To make a valid file name out of a complete path, we have to convert
+   or remove some characters that are significant to the filesystem -- in
+   particular, the path separator and any leading C{'.'} character (which would
+   cause the file to be hidden in a file listing).  
+
+   Note that this is a one-way transformation -- you can't safely derive the
+   original path from the normalized path.
+
+   To normalize a path, we begin by looking at the first character.  If the
+   first character is C{'/'} or C{'\\'}, it gets removed.  If the first
+   character is C{'.'}, it gets converted to C{'_'}.  Then, we look through the
+   rest of the path and convert all remaining C{'/'} or C{'\\'} characters
+   C{'-'}, and all remaining whitespace characters to C{'_'}.
+
+   As a special case, a path consisting only of a single C{'/'} or C{'\\'}
+   character will be converted to C{'-'}.
 
    @param path: Path to normalize
 
-   @return: Normalized path.
+   @return: Normalized path as described above.
+
+   @raise ValueError: If the path is None
    """
-   if path == os.sep:
+   if path is None:
+      raise ValueError("Cannot normalize path None.")
+   elif len(path) == 0:
+      return path
+   elif path == "/" or path == "\\":
       return "-"
    else:
       normalized = path
-      normalized = re.sub("^\.", "", normalized)
-      normalized = re.sub("^\/", "", normalized)
-      normalized = re.sub("\/", "-", normalized)
-      normalized = re.sub("\s", "_", normalized)
+      normalized = re.sub(r"^\/", "", normalized)  # remove leading '/'
+      normalized = re.sub(r"^\\", "", normalized)  # remove leading '\'
+      normalized = re.sub(r"^\.", "_", normalized) # convert leading '.' to '_' so file won't be hidden
+      normalized = re.sub(r"\/", "-", normalized)  # convert all '/' characters to '-'
+      normalized = re.sub(r"\\", "-", normalized)  # convert all '\' characters to '-'
+      normalized = re.sub(r"\s", "_", normalized)  # convert all whitespace to '_'
       return normalized
+
